﻿using Microsoft.Extensions.Logging;
using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.BusinessLogicsContracts;
using PlumbingRepairContracts.SearchModels;
using PlumbingRepairContracts.ViewModels;
using PlumbingRepairDataModels.Models;
using PlumbingRepairListImplement.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PlumbingRepairView
{
    public partial class FormStore : Form
    {
        private readonly List<StoreViewModel>? _listStores;
        private readonly IStoreLogic _logic;
        private readonly ILogger _logger;
        private Dictionary<int, (IWorkModel, int)> _works;
        public int Id { get; set; }

        public FormStore(ILogger<FormStore> logger, IStoreLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _listStores = logic.ReadList(null);
            _logic = logic;
            _works = new Dictionary<int, (IWorkModel, int)>();
        }

        private IStoreModel? GetStore(int id)
        {
            if (_listStores == null)
            {
                return null;
            }
            foreach (var elem in _listStores)
            {
                if (elem.Id == id)
                {
                    return elem;
                }
            }
            return null;
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(NameTextBox.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (string.IsNullOrEmpty(AdressTextBox.Text))
            {
                MessageBox.Show("Заполните адрес", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Сохранение изделия");

            try
            {
                DateTime.TryParse(OpeningDatePicker.Text, out var dateTime);
                StoreBindingModel model = new()
                {
                    StoreName = NameTextBox.Text,
                    StoreAdress = AdressTextBox.Text,
                    OpeningDate = dateTime,
                    Works = _works
                };
                var vmodel = GetStore(Id);
                bool operationResult = false;

                if (vmodel != null)
                {
                    model.Id = vmodel.Id;
                    operationResult = _logic.Update(model);
                }
                else
                {
                    operationResult = _logic.Create(model);
                }

                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения изделия");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void FormStore_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Загрузка изделия");
            try
            {
                var view = _logic.ReadElement(new StoreSearchModel { Id = Id });
                if (view != null)
                {
                    NameTextBox.Text = view.StoreName;
                    AdressTextBox.Text = view.StoreAdress;
                    OpeningDatePicker.Value = view.OpeningDate;
                    _works = view.Works ?? new Dictionary<int, (IWorkModel, int)>();
                    LoadData();
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки магазина");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            }
        private void LoadData()
        {
        try
            {
                if (_works != null)
                {
                    DataGridView.Rows.Clear();
                    foreach (var pc in _works)
                    {
                        DataGridView.Rows.Add(new object[] { pc.Key, pc.Value.Item1.WorkName, pc.Value.Item2 });
                    }
                }
            _logger.LogInformation("Загрузка магазинов");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки магазинов");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                MessageBoxIcon.Error);
            }
        }

        private void NameComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            LoadData();
        }
    }
}
